//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include <fstream>
#include "parallel/PeriodicMap.h"

namespace AMDiS {

  void PeriodicMap::add(const FiniteElemSpace *feSpace,
			PeriodicDofMap &newMap)
  {
    FUNCNAME("PeriodicMap::add()");
    
    for (PeriodicDofMap::iterator it = newMap.begin(); it != newMap.end(); ++it)
      for (DofMap::iterator dofIt =it->second.begin();
	   dofIt != it->second.end(); ++dofIt)
	add(feSpace, it->first, dofIt->second, dofIt->first);
  }


  void PeriodicMap::serialize(ostream &out, 
			      vector<const FiniteElemSpace*> feSpaces)
  {
    FUNCNAME("PeriodicMap::serialize()");
    
    int nFeSpace = static_cast<int>(feSpaces.size());
    
    for (int i = 0; i < nFeSpace; i++)
      serialize(out, periodicDofMap[feSpaces[i]]);
    
    for (int i = 0; i < nFeSpace; i++)
      serialize(out, periodicDofAssociations[feSpaces[i]]);
  }


  void PeriodicMap::deserialize(istream &in, 
				vector<const FiniteElemSpace*> feSpaces)
  {      
    FUNCNAME("PeriodicMap::deserialize()");
    
    int nFeSpace = static_cast<int>(feSpaces.size());
    
    for (int i = 0; i < nFeSpace; i++)
      deserialize(in, periodicDofMap[feSpaces[i]]);
    
    for (int i = 0; i < nFeSpace; i++)
      deserialize(in, periodicDofAssociations[feSpaces[i]]);
  }
  
  
  void PeriodicMap::serialize(ostream &out, PeriodicDofMap &data)
  {
    int mapSize = data.size();
    SerUtil::serialize(out, mapSize);
    
    for (PeriodicDofMap::iterator it = data.begin(); it != data.end(); ++it) {
      int type = it->first;
      DofMap dofMap = it->second;
      
      SerUtil::serialize(out, type);
      SerUtil::serialize(out, dofMap);
    }
  }
  

  void PeriodicMap::serialize(ostream &out, std::map<int, std::set<int> >& data)
  {
    int mapSize = data.size();
    SerUtil::serialize(out, mapSize);
    
    for (std::map<int, std::set<int> >::iterator it = data.begin(); 
	 it != data.end(); ++it) {
      int dof = it->first;
      std::set<int> typeSet = it->second;
      
      SerUtil::serialize(out, dof);
      SerUtil::serialize(out, typeSet);
    }
  }
  
  
  void PeriodicMap::deserialize(istream &in, PeriodicDofMap &data)
  {
    data.clear();
    
    int mapSize = 0;
    SerUtil::deserialize(in, mapSize);
    
    for (int i = 0; i < mapSize; i++) {
      int type;
      DofMap dofMap;
      
      SerUtil::deserialize(in, type);
      SerUtil::deserialize(in, dofMap);
      
      data[type] = dofMap;
    }
  }
  
  
  void PeriodicMap::deserialize(istream &in, std::map<int, std::set<int> >& data)
  {
    data.clear();
    
    int mapSize = 0;
    SerUtil::deserialize(in, mapSize);
    
    for (int i = 0; i < mapSize; i++) {
      int dof;
      std::set<int> typeSet;
      
      SerUtil::deserialize(in, dof);
      SerUtil::deserialize(in, typeSet);
      
      data[dof] = typeSet;
    }
  }
  
}
